//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.CustomDrawElements
{
	using CustomDrawElements = MindFusion.Scheduling.WinForms.CustomDrawElements;

	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		private class ItemSorter : IComparer
		{
			/// <summary>
			/// IComparer.Compare implementation.
			/// </summary>
			public int Compare(object x, object y)
			{
				ListViewItem xx = x as ListViewItem;
				ListViewItem yy = y as ListViewItem;

				return string.Compare(xx.Text, yy.Text);
			}
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			// Initialize the list with the values from the CustomDrawElements enum
			Array values = Enum.GetValues(typeof(CustomDrawElements));
			foreach (CustomDrawElements value in values)
			{
				if (value == CustomDrawElements.All ||
					value == CustomDrawElements.None)
					continue;

				ListViewItem item = new ListViewItem(value.ToString());
				item.Tag = value;
				list.Items.Add(item);
			}

			list.ListViewItemSorter = new ItemSorter();
			list.Sort();

			processing = false;

			calendar.MonthSettings.WeekHeaderStyle = MonthWeekHeaderStyle.Left;
			calendar.ListViewSettings.CellUnits = TimeUnit.Week;
			calendar.ListViewSettings.HeaderStyle |= ListViewHeaderStyles.Subheader;
			calendar.ResourceViewSettings.ViewStyle = ResourceViewStyle.Lanes;
			calendar.ResourceViewSettings.BottomTimelineSettings.Unit = TimeUnit.Hour;
			calendar.ResourceViewSettings.BottomTimelineSettings.Format = "HH:mm";
			calendar.TimetableSettings.ShowInfoHeader = State.Enabled;
			calendar.TimetableSettings.VisibleColumns = 3;
			calendar.Draw += new EventHandler<DrawEventArgs>(calendar_Draw);

			// Add some resources
			Resource r1 = new Resource();
			r1.Name = "Resource #1";
			calendar.Resources.Add(r1);

			Resource r2 = new Resource();
			r2.Name = "Resource #2";
			calendar.Resources.Add(r2);

			calendar.GroupType = GroupType.GroupByResources;

			// Add some items
			Appointment a;

			a = new Appointment();
			a.StartTime = new DateTime(2005, 9, 5);
			a.EndTime = new DateTime(2005, 9, 8);
			a.HeaderText = "Item #1";
			calendar.Schedule.Items.Add(a);

			a = new Appointment();
			a.StartTime = new DateTime(2005, 9, 7);
			a.EndTime = new DateTime(2005, 9, 9);
			a.HeaderText = "Item #2";
			calendar.Schedule.Items.Add(a);

			// Add some items associated with resources
			a = new Appointment();
			a.StartTime = new DateTime(2005, 9, 1, 4, 0, 0);
			a.EndTime = new DateTime(2005, 9, 1, 6, 30, 0);
			a.HeaderText = "Item #3";
			a.Resources.Add(r1);
			calendar.Schedule.Items.Add(a);

			a = new Appointment();
			a.StartTime = new DateTime(2005, 9, 1, 6, 0, 0);
			a.EndTime = new DateTime(2005, 9, 1, 10, 0, 0);
			a.HeaderText = "Item 43";
			a.Resources.Add(r1);
			calendar.Schedule.Items.Add(a);

			a = new Appointment();
			a.StartTime = new DateTime(2005, 9, 1);
			a.EndTime = new DateTime(2005, 9, 2);
			a.HeaderText = "Item #5";
			a.Resources.Add(r2);
			calendar.Schedule.Items.Add(a);
		}

		/// <summary>
		/// Handles the Draw event of the Calendar class.
		/// </summary>
		private void calendar_Draw(object sender, DrawEventArgs e)
		{
			Pen pen = new Pen(Color.DarkGoldenrod);
			Brush brush = new SolidBrush(Color.FromArgb(75, Color.Goldenrod));

			Rectangle bounds = e.Bounds;
			bounds.Width -= 1;
			bounds.Height -= 1;

			e.Graphics.FillRectangle(brush, bounds);
			e.Graphics.DrawRectangle(pen, bounds);

			brush.Dispose();
			pen.Dispose();
		}

		/// <summary>
		/// Handles the ItemCheck event of the list.
		/// </summary>
		private void list_ItemCheck(object sender, ItemCheckEventArgs e)
		{
			if (processing)
				return;
			processing = true;

			// Ensure only one item is selected
			if (e.NewValue == CheckState.Checked)
			{
				foreach (ListViewItem item in list.Items)
				{
					if (item == list.Items[e.Index])
						continue;

					item.Checked = false;
				}
			}

			// If no item is selected, reset the view
			if (e.NewValue != CheckState.Unchecked)
			{
				CustomDrawElements selected = (CustomDrawElements)list.Items[e.Index].Tag;
				switch (selected)
				{

					case CustomDrawElements.CalendarItem:
					case CustomDrawElements.CellContents:
					case CustomDrawElements.CellHeader:
						// The current view does not change
						break;

					case CustomDrawElements.ListViewHeader:
					case CustomDrawElements.ListViewHeaderCell:
					case CustomDrawElements.ListViewSubHeader:
					case CustomDrawElements.ListViewSubHeaderCell:
						calendar.CurrentView = CalendarView.List;
						break;

					case CustomDrawElements.MonthContents:
					case CustomDrawElements.MonthMainHeader:
					case CustomDrawElements.MonthRangeHeader:
					case CustomDrawElements.MonthSubHeader:
					case CustomDrawElements.MonthSubHeaderCell:
					case CustomDrawElements.MonthWeekNumbersHeader:
					case CustomDrawElements.MonthWeekNumbersHeaderCell:
						calendar.CurrentView = CalendarView.MonthRange;
						break;

					case CustomDrawElements.ResourceViewCell:
					case CustomDrawElements.ResourceViewCellComplete:
					case CustomDrawElements.ResourceViewItem:
					case CustomDrawElements.ResourceViewRowHeader:
					case CustomDrawElements.ResourceViewTimeline:
					case CustomDrawElements.ResourceViewTimelineCell:
						calendar.CurrentView = CalendarView.ResourceView;
						break;

					case CustomDrawElements.TimetableItem:
					case CustomDrawElements.TimetableCell:
					case CustomDrawElements.TimetableColumnHeader:
					case CustomDrawElements.TimetableGroupHeader:
					case CustomDrawElements.TimetableGroupHeaderCell:
					case CustomDrawElements.TimetableInfoHeader:
					case CustomDrawElements.TimetableTimeline:
					case CustomDrawElements.TimetableTimelineCell:
					case CustomDrawElements.TimetableTimelineHourCell:
					case CustomDrawElements.TimetableWholeDayHeader:
						calendar.CurrentView = CalendarView.Timetable;
						break;

					case CustomDrawElements.WeekRangeHeader:
					case CustomDrawElements.WeekRangeResourceHeader:
					case CustomDrawElements.WeekRangeSubHeader:
					case CustomDrawElements.WeekRangeSubHeaderCell:
						calendar.CurrentView = CalendarView.WeekRange;
						break;

				}

				calendar.CustomDraw = selected;
			}
			else
			{
				calendar.CustomDraw = CustomDrawElements.None;
			}

			calendar.Invalidate();

			processing = false;
		}


		private string _appPath;
		private bool processing;
	}
}
